package com.gic.plug.web.controller.goods;

import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.goods.api.dto.GoodsCordDTO;
import com.gic.goods.api.dto.SpecCordDTO;
import com.gic.goods.api.service.CordApiService;
import com.gic.goods.api.service.MallGoodsBusinessApiService;
import com.gic.goods.api.util.Constant;
import com.gic.mall.base.api.enums.ErrorCodeEnums;
import com.gic.mall.share.api.dto.shop.ShopDTO;
import com.gic.mall.share.api.service.ShopApiService;
import com.gic.member.config.api.dto.AppletsConfigSimpleOutputDTO;
import com.gic.member.config.api.service.AppletsConfigApiService;
import com.gic.plug.web.vo.goods.MallGoodsCordVO;
import com.gic.search.business.api.constant.enums.OperateEnum;
import com.gic.search.business.api.dto.DynamicSearchDTO;
import com.gic.search.business.api.dto.ESResponseQueryBatchDTO;
import com.gic.search.business.api.service.EsBusinessOperaApiService;
import com.gic.search.business.api.utils.QueryConditionAssemblyUtil;
import com.gic.widget.screening.api.service.EsScreeningSearchService;
import com.google.common.base.Joiner;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.curator.shaded.com.google.common.base.Splitter;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 商城装修选择器
 */
@RestController
public class AppletConfigGoodsController extends BaseGoodsController {

    @Autowired
    private CordApiService cordApiService;

    @Autowired
    private ShopApiService shopApiService;

    @Autowired
    private AppletsConfigApiService appletsConfigApiService;

    @Autowired
    private EsScreeningSearchService esScreeningSearchService;

    @Autowired
    private EsBusinessOperaApiService esBusinessOperaApiService;

    @Autowired
    private MallGoodsBusinessApiService mallGoodsBusinessApiService;


    private static final Logger log = LogManager.getLogger(AppletConfigGoodsController.class);

    @RequestMapping("get-mall-cord-list")
    public RestResponse getMallCordList(String appConfigId, Long shopId) {
        Long areaId = null;
        if (StringUtils.isNotBlank(appConfigId)) {
            ServiceResponse<AppletsConfigSimpleOutputDTO> appServiceResponse = appletsConfigApiService.selectAppletsConfigIdByAppId(appConfigId, getEnterpriseId());
            if (appServiceResponse.getResult() == null) {
                return RestResponse.failure(ErrorCodeEnums.MAll_ERR.getCode(), "小程序不存在");
            }
            ServiceResponse<ShopDTO> shopDTOServiceResponse = shopApiService.getShopByAppletId(appServiceResponse.getResult().getAppletConfigId(), getEnterpriseId(), 1);
            if (shopDTOServiceResponse.getResult() == null) {
                return RestResponse.failure(ErrorCodeEnums.MAll_ERR.getCode(), "不存在店铺");
            }
            areaId = shopDTOServiceResponse.getResult().getAreaId();
        } else if (shopId != null) {
            ServiceResponse<ShopDTO> shopDTOServiceResponse = shopApiService.getShopDetailsById(shopId, getEnterpriseId());
            if (shopDTOServiceResponse.getResult() == null) {
                return RestResponse.failure(ErrorCodeEnums.MAll_ERR.getCode(), "不存在店铺");
            }
            areaId = shopDTOServiceResponse.getResult().getAreaId();
        }
        ServiceResponse<List<SpecCordDTO>> listServiceResponse = cordApiService.listSpecCordDTO(areaId, getEnterpriseId());
        return RestResponse
                .success(EntityUtil.changeEntityListByOrika(MallGoodsCordVO.class, listServiceResponse.getResult()));
    }

    /**
     * 商城组对外提供的组件
     *
     * @param currentPage
     * @param pageSize
     * @param appConfigId
     * @param search
     * @param cordId
     * @return
     */
    @RequestMapping("list-mall-goods-selector")
    public RestResponse listGoodsByChannel(@RequestParam(value = "currentPage", defaultValue = "1") Integer currentPage,
                                           @RequestParam(value = "pageSize", defaultValue = "20") Integer pageSize, String appConfigId, String search, Long cordId,
                                           Long shopId) {
        Long areaId = null;
        if (StringUtils.isNotBlank(appConfigId)) {
            ServiceResponse<AppletsConfigSimpleOutputDTO> appServiceResponse = appletsConfigApiService.selectAppletsConfigIdByAppId(appConfigId, getEnterpriseId());
            if (appServiceResponse.getResult() == null) {
                return RestResponse.failure(ErrorCodeEnums.MAll_ERR.getCode(), "小程序不存在");
            }
            Integer shopType = 1;
            ServiceResponse<ShopDTO> shopDTOServiceResponse = shopApiService.getShopByAppletId(appServiceResponse.getResult().getAppletConfigId(), getEnterpriseId(), shopType);
            if (shopDTOServiceResponse.getResult() == null) {
                return RestResponse.failure(ErrorCodeEnums.MAll_ERR.getCode(), "不存在店铺");
            }
            areaId = shopDTOServiceResponse.getResult().getAreaId();
        } else if (shopId != null) {
            ServiceResponse<ShopDTO> shopDTOServiceResponse = shopApiService.getShopDetailsById(shopId, getEnterpriseId());
            if (shopDTOServiceResponse.getResult() == null) {
                return RestResponse.failure(ErrorCodeEnums.MAll_ERR.getCode(), "不存在店铺");
            }
            areaId = shopDTOServiceResponse.getResult().getAreaId();
        }
        JSONObject cordJSON = null;
        if (cordId != null) {
            ServiceResponse<List<GoodsCordDTO>> serviceResponse = cordApiService.listChildAll(cordId, getEnterpriseId());
            List<Long> cordList = new ArrayList<>();
            for (GoodsCordDTO goodsCord : serviceResponse.getResult()) {
                cordList.add(goodsCord.getSpecCordId());
            }
            cordList.add(cordId);
            cordJSON = QueryConditionAssemblyUtil.createSimpleQueryNode("cordId",
                    OperateEnum.OPERATE_CONTAIN, Joiner.on(" ").join(cordList));
        }
        ServiceResponse<ESResponseQueryBatchDTO> serviceResponse = searchGoods(currentPage, pageSize, areaId, search,
                Constant.CHANNEL_CODE_MALL, cordJSON);
        if (serviceResponse.isSuccess()) {
            List<Long> goodsIds = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(serviceResponse.getResult().getRes())) {
                serviceResponse.getResult().getRes().forEach(json -> {
                    Long goodsId = json.getLong("goodsId");
                    goodsIds.add(goodsId);
                });
            }
            ServiceResponse<Map<Long, BigDecimal>> mapServiceResponse = mallGoodsBusinessApiService.getLowestSkuPrice(goodsIds, getEnterpriseId());
            if (mapServiceResponse.isSuccess()) {
                Map<Long, BigDecimal> map = mapServiceResponse.getResult();
                serviceResponse.getResult().getRes().forEach(json -> {
                    Long goodsId = json.getLong("goodsId");
                    json.put("minSkuPrice", map.getOrDefault(goodsId, BigDecimal.valueOf(0)));
                });
            }
            return RestResponse.success(serviceResponse.getResult());
        }
        return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
    }

    /**
     * 商城组对内的渠道商品组件
     *
     * @param currentPage
     * @param pageSize
     * @param search
     * @return
     */
    @RequestMapping("list-channel-goods-selector")
    public RestResponse listGoodsChannel(@RequestParam(value = "currentPage", defaultValue = "1") Integer currentPage,
                                         @RequestParam(value = "pageSize", defaultValue = "20") Integer pageSize, String search,
                                         Double startPrice, Double endPrice, Integer startStock, Integer endStock, String channelCode) {
        JSONObject queryObject = null;
        if (StringUtils.isBlank(channelCode)) {
            channelCode = Constant.CHANNEL_CODE_MALL;
        }
        if (startPrice != null) {
            queryObject = QueryConditionAssemblyUtil.createSimpleQueryNode("goodsPrice",
                    OperateEnum.OPERATE_GTE, startPrice);
        }
        if (endPrice != null) {
            if (queryObject == null) {
                queryObject = QueryConditionAssemblyUtil.createSimpleQueryNode("goodsPrice",
                        OperateEnum.OPERATE_LTE, endPrice);
            } else {
                queryObject = QueryConditionAssemblyUtil.addQueryNodeForSiblin("goodsPrice",
                        OperateEnum.OPERATE_LTE, endPrice, queryObject, OperateEnum.OPERATE_AND);
            }
        }
        if (startStock != null) {
            if (queryObject == null) {
                queryObject = QueryConditionAssemblyUtil.createSimpleQueryNode("stock",
                        OperateEnum.OPERATE_GTE, startStock);
            } else {
                queryObject = QueryConditionAssemblyUtil.addQueryNodeForSiblin("stock",
                        OperateEnum.OPERATE_GTE, startStock, queryObject, OperateEnum.OPERATE_AND);
            }
        }
        if (endStock != null) {
            if (queryObject == null) {
                queryObject = QueryConditionAssemblyUtil.createSimpleQueryNode("stock",
                        OperateEnum.OPERATE_LTE, endStock);
            } else {
                queryObject = QueryConditionAssemblyUtil.addQueryNodeForSiblin("stock",
                        OperateEnum.OPERATE_LTE, endStock, queryObject, OperateEnum.OPERATE_AND);
            }
        }
        ServiceResponse<ESResponseQueryBatchDTO> serviceResponse = searchGoods(currentPage, pageSize, null, search,
                channelCode, queryObject);
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        }
        return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
    }


    private Long getGoodsDomainIdByAponfigId(String appConfigId) {
        Long goodsDomainId = null;
        ServiceResponse<AppletsConfigSimpleOutputDTO> appServiceResponse = appletsConfigApiService.selectAppletsConfigIdByAppId(appConfigId, getEnterpriseId());
        if (appServiceResponse.getResult() == null) {
            log.info("查不到小程序信息 appConfigId={}", appConfigId);
        }
        Integer shopType = 2;
        ServiceResponse<ShopDTO> shopDTOServiceResponse = shopApiService.getShopByAppletId(appServiceResponse.getResult().getAppletConfigId(), getEnterpriseId(), shopType);
        if (shopDTOServiceResponse.getResult() == null) {

            log.info("查不到店铺信息 appConfigId={}", appConfigId);
            return null;
        }
        goodsDomainId = shopDTOServiceResponse.getResult().getAreaId();
        return goodsDomainId;

    }

    private ServiceResponse<ESResponseQueryBatchDTO> searchGoods(Integer currentPage, Integer pageSize,
                                                                 Long goodsDomainId, String search, String channelCode, JSONObject queryObject) {
        DynamicSearchDTO searchDTO = new DynamicSearchDTO();
        String searchCode = "";
        if (Constant.CHANNEL_CODE_MALL.equals(channelCode)) {
            searchCode = "gic_mall_goods";
        } else if (Constant.CHANNEL_CODE_ERP.equals(channelCode)) {
            searchCode = "erp_goods";
        }
        String enterpriseId = getEnterpriseId() + "";
        String esName = esScreeningSearchService.getCurrentEsName(enterpriseId, searchCode);
        searchDTO.setIndexName(esName);
        searchDTO.setColumnCategoryCode(searchCode);
        searchDTO.setEnterpriseId(enterpriseId);
        searchDTO.setType("mapper_type");
        searchDTO.setBegin((currentPage - 1) * pageSize);
        searchDTO.setRecordNumber(pageSize);
        JSONObject jsonObject = null;
        JSONObject status = QueryConditionAssemblyUtil.createSimpleQueryNode("status", OperateEnum.OPERATE_EQ, 2);
        if (Constant.CHANNEL_CODE_MALL.equals(channelCode)) {
            status = QueryConditionAssemblyUtil.addQueryNodeForSiblin(status, QueryConditionAssemblyUtil.createSimpleQueryNode("upShelfTime", OperateEnum.OPERATE_LTE, new Date().getTime()), OperateEnum.OPERATE_AND);
        }
        if (goodsDomainId == null) {
            jsonObject = status;
        } else {
            jsonObject = QueryConditionAssemblyUtil.createSimpleQueryNode("goodsDomainId",
                    OperateEnum.OPERATE_EQ, goodsDomainId);
            jsonObject = QueryConditionAssemblyUtil.addQueryNodeForSiblin(status, jsonObject, OperateEnum.OPERATE_AND);
        }
        if (StringUtils.isNotBlank(search)) {
            JSONObject goodsCodeJson = QueryConditionAssemblyUtil.createSimpleQueryNode("goodsCode",
                    OperateEnum.OPERATE_LIKE, search);
            JSONObject goodsNameJson = QueryConditionAssemblyUtil.createSimpleQueryNode("goodsName",
                    OperateEnum.OPERATE_LIKE, search);
            JSONObject goodsBarCodeJson = QueryConditionAssemblyUtil.createSimpleQueryNode("goodsBarCode",
                    OperateEnum.OPERATE_LIKE, search);
            List<JSONObject> jsonArray = new ArrayList<>();
            jsonArray.add(goodsCodeJson);
            jsonArray.add(goodsNameJson);
            jsonArray.add(goodsBarCodeJson);
            JSONObject object = QueryConditionAssemblyUtil.createListQueryNode(jsonArray, OperateEnum.OPERATE_OR);
            jsonObject = QueryConditionAssemblyUtil.addQueryNodeForSiblin(object, jsonObject, OperateEnum.OPERATE_AND);
        }
        if (queryObject != null) {
            jsonObject = QueryConditionAssemblyUtil.addQueryNodeForSiblin(queryObject, jsonObject, OperateEnum.OPERATE_AND);
        }
        searchDTO.setSearchJson(jsonObject);
        ServiceResponse<ESResponseQueryBatchDTO> serviceResponse = esBusinessOperaApiService.queryDataBatch(searchDTO,
                false, null);
        return serviceResponse;
    }


    /**
     * 小程序积分商城查询
     *
     * @param currentPage
     * @param pageSize
     * @param appConfigId
     * @param cordId
     * @return 调试接口
     */
    //@RequestMapping("list-integral-goods-selector")
    public RestResponse listIntegralGoodsByChannel(@RequestParam(value = "currentPage", defaultValue = "1") Integer currentPage,
                                                   @RequestParam(value = "pageSize", defaultValue = "20") Integer pageSize, String appConfigId, Long cordId,String sortColumn,String ids,Integer proType) {

        Long goodsDomainId = getGoodsDomainIdByAponfigId(appConfigId);
        DynamicSearchDTO searchDTO = new DynamicSearchDTO();
        String columnCategoryCode = "gic_integral_goods";
        String esName = esScreeningSearchService.getCurrentEsName(getEnterpriseId() + "", columnCategoryCode);
        searchDTO.setIndexName(esName);
        searchDTO.setColumnCategoryCode(columnCategoryCode);
        searchDTO.setEnterpriseId(getEnterpriseId() + "");
        searchDTO.setType("mapper_type");
        searchDTO.setBegin((currentPage - 1) * pageSize);
        searchDTO.setRecordNumber(pageSize);
        JSONObject sortNode = QueryConditionAssemblyUtil.createSortNode(sortColumn, "desc");
        List<JSONObject> jsonArray = new ArrayList<>();
        JSONObject proTypejson = QueryConditionAssemblyUtil.createSimpleQueryNode("proType", OperateEnum.OPERATE_EQ, proType);
        JSONObject goodsDomainIdJson = QueryConditionAssemblyUtil.createSimpleQueryNode("goodsDomainId", OperateEnum.OPERATE_EQ, goodsDomainId);
        JSONObject  json = QueryConditionAssemblyUtil.addQueryNodeForSiblin(proTypejson, goodsDomainIdJson, OperateEnum.OPERATE_AND);
        JSONObject upShelvesTimejson = QueryConditionAssemblyUtil.createSimpleQueryNode("upShelvesTime",  OperateEnum.OPERATE_LTE, System.currentTimeMillis());
        json = QueryConditionAssemblyUtil.addQueryNodeForSiblin(json, upShelvesTimejson, OperateEnum.OPERATE_AND);
        if (cordId != null) {
            JSONObject goodsCodeJson = QueryConditionAssemblyUtil.createSimpleQueryNode("goodsCordId", OperateEnum.OPERATE_EQ, cordId);
            json = QueryConditionAssemblyUtil.addQueryNodeForSiblin(json, goodsCodeJson, OperateEnum.OPERATE_AND);
            jsonArray.add(goodsCodeJson);
        }
        if (ids != null) {
            List<Long> collect = Splitter.on(",").splitToList(ids).stream().map(x -> Long.parseLong(x)).collect(Collectors.toList());
            JSONObject   goodsIdsJSON = QueryConditionAssemblyUtil.createSimpleQueryNode("id", OperateEnum.OPERATE_CONTAIN, Joiner.on(" ").join(collect));
            JSONObject strategyJson = QueryConditionAssemblyUtil.createSimpleQueryNode("strategyFlag", OperateEnum.OPERATE_EQ, 0);
            JSONObject strategyAndgoodsIdsJSON = QueryConditionAssemblyUtil.addQueryNodeForSiblin(goodsIdsJSON, strategyJson, OperateEnum.OPERATE_OR);
            json = QueryConditionAssemblyUtil.addQueryNodeForSiblin(json, strategyAndgoodsIdsJSON, OperateEnum.OPERATE_AND);
            jsonArray.add(strategyAndgoodsIdsJSON);
        }else{
            JSONObject strategyJson = QueryConditionAssemblyUtil.createSimpleQueryNode("strategyFlag", OperateEnum.OPERATE_EQ, 0);
            json = QueryConditionAssemblyUtil.addQueryNodeForSiblin(json, strategyJson, OperateEnum.OPERATE_AND);
            jsonArray.add(strategyJson);
        }
        searchDTO.setSearchJson(json);
        searchDTO.setSortJson(sortNode);
        log.info("查询条件 = {}",JSONObject.toJSONString(searchDTO));
        ServiceResponse<ESResponseQueryBatchDTO> serviceResponse = esBusinessOperaApiService.queryDataBatch(searchDTO, false, null);
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        }
        return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
    }

}
